
/*----------------------------------------------------------
---*\
 |   CUSTRES.C - Creating a custom resource in Windows.  This  |
 |               program creates a resource that contains      |
 |               a Sin and Cosine table.  It uses this table   |
 |               to calculate from both degrees and radians.   |
\*----------------------------------------------------------
---*/
#include <Windows.H>
#include "custres.h"

/*----------------------------------------------------------
---*\
 |                    Function Prototypes.                     |
\*----------------------------------------------------------
---*/
long FAR  PASCAL MinWndProc (HWND, unsigned, WORD, LONG);
int NEAR PASCAL intSin (int iValue);
int NEAR PASCAL intCos (int iValue);

/*----------------------------------------------------------
---*\
 |                  Static Data Allocation.                    |
\*----------------------------------------------------------
---*/
HANDLE hInst;
HANDLE hresSinData;

/*----------------------------------------------------------
---*\
 |                 Main Function:  WinMain.                    |
\*----------------------------------------------------------
---*/
int PASCAL WinMain (HANDLE hInstance,
                    HANDLE hPrevInstance,
                    LPSTR  lpszCmdLine,
                    int    cmdShow)
    {
    HWND     hwnd;
    MSG      msg;
    WNDCLASS wndclass;

    if (!hPrevInstance)
        {
        wndclass.lpszClassName = "MIN:MAIN";
        wndclass.hInstance     = hInstance;
        wndclass.lpfnWndProc   = MinWndProc;
        wndclass.hCursor       = LoadCursor (hInstance, "hand");
        wndclass.hIcon         = LoadIcon (hInstance,"snapshot");
        wndclass.lpszMenuName  = NULL;
        wndclass.hbrBackground = COLOR_WINDOW+1;
        wndclass.style         = CS_HREDRAW | CS_VREDRAW;
        wndclass.cbClsExtra    = 0;
        wndclass.cbWndExtra    = 0;

        RegisterClass( &wndclass);
        }

    hInst = hInstance;

    hwnd = CreateWindow("MIN:MAIN",           /* Class name.   */
                                              /* Title.        */
                        "Custom Resource - Sin & Cosine",
                        WS_OVERLAPPEDWINDOW,  /* Style bits.   */
                        CW_USEDEFAULT,        /* x - default.  */
                        0,                    /* y - default.  */
                        CW_USEDEFAULT,        /* cx - default. */
                        0,                    /* cy - default. */
                        NULL,                 /* No parent.    */
                        NULL,                 /* Class menu.   */
                        hInstance,            /* Creator       */
                        NULL                  /* Params.       */
                       ) ;
    ShowWindow (hwnd, cmdShow);
    while (GetMessage(&msg, 0, 0, 0))
        {
        TranslateMessage(&msg);       /*  Keyboard input.      */
        DispatchMessage(&msg);
        }
    return 0;
    }

/*----------------------------------------------------------
---*\
 |              Window Procedure:  MinWndProc.                 |
\*----------------------------------------------------------
---*/
long FAR PASCAL MinWndProc (HWND hwnd,   WORD wMsg,
                            WORD wParam, LONG lParam)
    {

    switch (wMsg)
        {
        case WM_CREATE:
            {
            HANDLE hRes;
            hRes = FindResource (hInst,
                       MAKEINTRESOURCE(SIN),      /* Name.  */
                       MAKEINTRESOURCE(TABLE));   /* Type.  */
            hresSinData = LoadResource (hInst, hRes);
            }
            break;

        case WM_DESTROY:
            FreeResource (hresSinData);
            PostQuitMessage(0);
            break;

        case WM_PAINT:
            {
            int i;
            int x, y;
            PAINTSTRUCT ps;
            RECT r;

            BeginPaint (hwnd, &ps);
            GetClientRect (hwnd, &r);
            SetViewportOrg (ps.hdc, r.right/2, r.bottom/2);

            x = intCos (0)/100;
            y = intSin (0)/100;
            MoveTo (ps.hdc, x, y);

            for (i=0;i<=360;i++)
                {
                x = intCos (i)/100;
                y = intSin (i)/100;
                LineTo (ps.hdc, x, y);
                }

            EndPaint (hwnd, &ps);
            }
            break;

        default:
            return(DefWindowProc(hwnd,wMsg,wParam,lParam));
            break;
        }
    return 0L;
    }


/*----------------------------------------------------------
---*\
 |  intSin  -  Calculates an integer sin value in units equal  |
 |             to 10,000th for any degree entered, using a     |
 |             value derived from a custom sin resource.       |
\*----------------------------------------------------------
---*/
int NEAR PASCAL intSin (int iValue)
    {
    int iSign;

    int FAR * fpSin;

    fpSin = (int FAR *)LockResource (hresSinData);
    if (fpSin == NULL)
        return (0);

    while (iValue < 0)   iValue +=360;
    while (iValue > 360) iValue -=360;

    iSign = 1;

    if (iValue > 90 && iValue <=180)
        {
        iValue = 180 - iValue;
        }
    else if (iValue > 180 && iValue <= 270)
        {
        iSign = -1;
        iValue = iValue - 180;
        }
    else if (iValue > 270 && iValue <= 360)
        {
        iSign = -1;
        iValue = 360 - iValue;
        }

    /*  Adjust pointer to correct table entry.  */
    fpSin += iValue;

    iSign = *fpSin * iSign;
    UnlockResource (hresSinData);

    return (iSign);
    }


/*----------------------------------------------------------
---*\
 |  intCos  -  Calculates an integer cosine value in units     |
 |             equal to 10,000th for any degree entered, using |
 |             a value derived from a custom sin resource.     |
\*----------------------------------------------------------
---*/
int NEAR PASCAL intCos (int iValue)
    {
    return (intSin (iValue-90));
    }

