
/*----------------------------------------------------------
---*\
 |  SUBSEG - Sample program showing global heap allocation     |
 |             in Microsoft Windows.                           |
\*----------------------------------------------------------
---*/
#include <Windows.H>

typedef DWORD HANDLE32;

/*----------------------------------------------------------
---*\
 |                    Function Prototypes.                     |
\*----------------------------------------------------------
---*/
long FAR  PASCAL SubsegWndProc (HWND, unsigned, WORD, LONG);

BOOL     FAR PASCAL SubInitialize(VOID);
HANDLE32 FAR PASCAL SubAlloc(WORD, WORD);
HANDLE32 FAR PASCAL SubFree(HANDLE32);
LPSTR    FAR PASCAL SubLock(HANDLE32);
WORD     FAR PASCAL SubSize(HANDLE32);
BOOL     FAR PASCAL SubUnlock(HANDLE32);

/*----------------------------------------------------------
---*\
 |                 Main Function:  WinMain.                    |
\*----------------------------------------------------------
---*/
int PASCAL WinMain (HANDLE hInstance,
                    HANDLE hPrevInstance,
                    LPSTR  lpszCmdLine,
                    int    cmdShow)
    {
    HWND     hwnd;
    MSG      msg;
    WNDCLASS wndclass;

    if (!hPrevInstance)
        {
        wndclass.lpszClassName = "Subseg:MAIN";
        wndclass.hInstance     = hInstance;
        wndclass.lpfnWndProc   = SubsegWndProc;
        wndclass.hCursor       = LoadCursor (hInstance, "hand");
        wndclass.hIcon         = LoadIcon (hInstance,"snapshot");
        wndclass.lpszMenuName  = NULL;
        wndclass.hbrBackground = COLOR_WINDOW+1;
        wndclass.style         = NULL;
        wndclass.cbClsExtra    = 0;
        wndclass.cbWndExtra    = 0;

        RegisterClass( &wndclass);
        }

    hwnd = CreateWindow("Subseg:MAIN",        /* Class name.   */
                                              /* Title.        */
                        "Sub-Segment Allocation",
                        WS_OVERLAPPEDWINDOW,  /* Style bits.   */
                        CW_USEDEFAULT,        /* x - default.  */
                        0,                    /* y - default.  */
                        CW_USEDEFAULT,        /* cx - default. */
                        0,                    /* cy - default. */
                        NULL,                 /* No parent.    */
                        NULL,                 /* Class menu.   */
                        hInstance,            /* Creator       */
                        NULL                  /* Params.       */
                       ) ;
    ShowWindow (hwnd, cmdShow);

    while (GetMessage(&msg, 0, 0, 0))
        {
        TranslateMessage(&msg);       /*  Keyboard input.      */
        DispatchMessage(&msg);
        }
    return 0;
    }

#define STRSIZE 30
typedef struct tagSEGDATA
    {
    char  achDesc[STRSIZE];  /*  Description of data.  */
    DWORD dwAlloc;           /*  Amount asked for.     */
    DWORD dwActual;          /*  Actually allocated.   */
    WORD  wFlags;            /*  Allocation flags.     */
    } SEGDATA;

typedef SEGDATA FAR *LPSEGDATA;

#define COUNT      3

/*----------------------------------------------------------
---*\
 |              Window Procedure:  SubsegWndProc.              |
\*----------------------------------------------------------
---*/
long FAR PASCAL SubsegWndProc (HWND hwnd,   WORD wMsg,
                                 WORD wParam, LONG lParam)
    {
    static char Label1[] = "Description";
    static char Label2[] = "Req/Actual";
    static char Label3[] = "Handle  ->  Address";

    static int  cb1 = sizeof (Label1) - 1;
    static int  cb2 = sizeof (Label2) - 1;
    static int  cb3 = sizeof (Label3) - 1;

    static HANDLE32 h[COUNT];
    static SEGDATA sd[COUNT] =
        {
        "Object # 1 - Fixed",        50, 0, LMEM_FIXED,
        "Object # 2 - Moveable",     75, 0, LMEM_MOVEABLE |
                                            LMEM_ZEROINIT,
        "Object # 3 - Discardable", 100, 0, LMEM_DISCARDABLE |
                                            LMEM_MOVEABLE,
        };
    static TEXTMETRIC tmSys;

    switch (wMsg)
        {
        case WM_CREATE:
            {
            HDC hdc;
            int i;
            LPSEGDATA lpsd;

            if (!SubInitialize())
                goto ErrorExit;

            for (i=0;i<COUNT;i++)
                {
                h[i] = SubAlloc(sd[i].wFlags,
                                sd[i].dwAlloc);
                if (!h[i])
                    goto ErrorExit;

                lpsd = (LPSEGDATA)SubLock (h[i]);
                if (!lpsd)
                    goto ErrorExit;

                lstrcpy (lpsd->achDesc, sd[i].achDesc);
                lpsd->dwAlloc = sd[i].dwAlloc;
                lpsd->dwActual = SubSize (h[i]);
                lpsd->wFlags = sd[i].wFlags;
                SubUnlock (h[i]);
                } /* [for] */

            hdc = GetDC (hwnd);
            GetTextMetrics (hdc, &tmSys);
            ReleaseDC (hwnd, hdc);
            break;

ErrorExit:
            MessageBox (GetFocus(), "Not enough memory",
                        "Subseg", MB_OK);
            DestroyWindow (hwnd);
            }
            break;

        case WM_DESTROY:
            PostQuitMessage(0);
            break;

        case WM_PAINT:
            {
            char buff[30];
            int i;
            int xText1;
            int xText2;
            int xText3;
            int yText;

            LPSEGDATA lpsd;
            PAINTSTRUCT ps;

            BeginPaint (hwnd, &ps);

            /*  Calculate text positioning variables.          */
            xText1 = tmSys.tmAveCharWidth * 2;
            xText2 = xText1 + (STRSIZE * tmSys.tmAveCharWidth);
            xText3 = xText2 + ((cb2+5) * tmSys.tmAveCharWidth);
            yText = tmSys.tmHeight;

            /*  Print titles.                                  */
            TextOut (ps.hdc, xText1, yText, Label1, cb1);
            TextOut (ps.hdc, xText2, yText, Label2, cb2);
            TextOut (ps.hdc, xText3, yText, Label3, cb3);

            yText += tmSys.tmHeight * 2;

            for (i=0;i<COUNT;i++)
                {
                lpsd = (LPSEGDATA)SubLock (h[i]);

                /*  Print description.                         */
                TextOut (ps.hdc, xText1, yText,
                         lpsd->achDesc, lstrlen(lpsd->achDesc));

                /*  Print allocated vs actual size.            */
                wsprintf (buff, "%ld / %ld", lpsd->dwAlloc,
                                             lpsd->dwActual);
                TextOut (ps.hdc, xText2, yText,
                         buff, lstrlen (buff));

                /*  Print handle and actual address.           */
                wsprintf (buff, "%04x:%04x  ->  %04x:%04x",
                          HIWORD(h[i]), LOWORD(h[i]),
                          HIWORD(lpsd), LOWORD(lpsd));
                TextOut (ps.hdc, xText3, yText,
                         buff, lstrlen (buff));

                SubUnlock (h[i]);

                /*  Advance to next line.                      */
                yText += tmSys.tmHeight + tmSys.tmExternalLeading;
                }

            EndPaint (hwnd, &ps);
            }
            break;

        default:
            return(DefWindowProc(hwnd,wMsg,wParam,lParam));
            break;
        }
    return 0L;
    }

